;    awpop3.pbi
;    Version 0.6 [2013/08/13]
;    Copyright (C) 2011 Ronny Krueger
;
;    This file is part of AWPB-Tools.
;
;    AWPB-Tools is free software: you can redistribute it and/or modify
;    it under the terms of the GNU Lesser General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    AWPB-Tools is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU Lesser General Public License for more details.
;
;    You should have received a copy of the GNU Lesser General Public License
;    along with AWPB-Tools.  If not, see <http://www.gnu.org/licenses/>.
;
; Code based on and inspired by:
;
; {==============================================================================|
; | Project : Ararat Synapse                                       | 005.002.002 |
; |==============================================================================|
; | Content: Charset conversion support                                          |
; |==============================================================================|
; | Copyright (c)1999-2004, Lukas Gebauer                                        |
; | All rights reserved.                                                         |
; |                                                                              |
; | Redistribution And use in source And binary forms, With Or without           |
; | modification, are permitted provided that the following conditions are met:  |
; |                                                                              |
; | Redistributions of source code must retain the above copyright notice, this  |
; | List of conditions And the following disclaimer.                             |
; |                                                                              |
; | Redistributions in binary form must reproduce the above copyright notice,    |
; | this List of conditions And the following disclaimer in the documentation    |
; | And/Or other materials provided With the distribution.                       |
; |                                                                              |
; | Neither the name of Lukas Gebauer nor the names of its contributors may      |
; | be used To endorse Or promote products derived from this software without    |
; | specific prior written permission.                                           |
; |                                                                              |
; | THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS And CONTRIBUTORS "AS IS"  |
; | And ANY EXPRESS Or IMPLIED WARRANTIES, INCLUDING, BUT Not LIMITED To, THE    |
; | IMPLIED WARRANTIES OF MERCHANTABILITY And FITNESS For A PARTICULAR PURPOSE   |
; | ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS Or CONTRIBUTORS BE LIABLE For  |
; | ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, Or CONSEQUENTIAL       |
; | DAMAGES (INCLUDING, BUT Not LIMITED To, PROCUREMENT OF SUBSTITUTE GOODS Or   |
; | SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER   |
; | CAUSED And ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT           |
; | LIABILITY, Or TORT (INCLUDING NEGLIGENCE Or OTHERWISE) ARISING IN ANY WAY    |
; | OUT OF THE USE OF THIS SOFTWARE, EVEN If ADVISED OF THE POSSIBILITY OF SUCH  |
; | DAMAGE.                                                                      |
; |==============================================================================|
; | The Initial Developer of the Original Code is Lukas Gebauer (Czech Republic).|
; | Portions created by Lukas Gebauer are Copyright (c)2000-2004.                |
; | All Rights Reserved.                                                         |
; |==============================================================================|
; | Contributor(s):                                                              |
; |==============================================================================|
; | History: see HISTORY.HTM from distribution package                           |
; |          (Found at URL: http://www.ararat.cz/synapse/)                       |
; |==============================================================================}

CompilerIf Defined(Enable_OpenSSL, #PB_Constant)
  XIncludeFile "awopenssl.pbi"
CompilerEndIf

XIncludeFile "awnetwork.pbi"
XIncludeFile "awsupport.pbi"
XIncludeFile "awmessage.pbi"

;{ Constants
#AWPB_POP3_CONN_USER    = $0001
#AWPB_POP3_CONN_APOP    = $0002
#AWPB_POP3_CONN_TLS     = $0004
#AWPB_POP3_CONN_AUTO    = #AWPB_POP3_CONN_USER | #AWPB_POP3_CONN_APOP
#AWPB_POP3_CONN_AUTOTLS = #AWPB_POP3_CONN_AUTO | #AWPB_POP3_CONN_TLS

#AWPB_POP3_ERR_NONE     = 0
#AWPB_POP3_ERR_FAIL     = -1
#AWPB_POP3_ERR_LOGIN    = -2
#AWPB_POP3_ERR_USER     = -3
#AWPB_POP3_ERR_PASS     = -4

;}
;{ POP3-Object
Interface AWPB_POP3Client
  ; public
  Destroy            ()
  NOOP.i             ()
  RSET.i             ()
  Connect.i          (flags.i = #AWPB_POP3_CONN_AUTO)
  GetServerResult.s  ()
  SetServer          (Host.s, Port.i = 110)
  SetLogin           (User.s, Pass.s)
  SetTimeOut         (TimeOut.i = 15000)
  DisConnect         ()
  GetMailCount.i     ()
  GetMailHeaderStr.s (index.i)
  GetMailHeader.i    (index.i)
  GetMailStr         (index.i)
  GetMail            (index.i)
  GetLog.s					 ()
  ClearLog					 ()
  EnableLog					 ()
  DisableLog				 ()

  ; private
  _CAPA.s            ()
  _LIST.s            (index.i)
  _QUIT.i            ()
  _USER.i            ()
  _PASS.i            ()
  _Login.i           ()
  _APOP.i            ()
  _STLS.i            ()
  _DELE.i            (index.i)
  _STAT.i            ()
  _TOP.s             (index.i)
  _RETR.s            (index.i)

EndInterface
Structure ____AWPB_POP3Client
  ; private
    *vt.AWPB_POP3Client
    *Conn.____NW_NetworkConn
    Caps.s
    TimeStamp.s
  ; public

EndStructure
;{ Member Procedures
Procedure.i AWPB_CreatePOP3ClientObj(EnableLog.i = #False)
  Protected *this.____AWPB_POP3Client

  *this = AllocateMemory(SizeOf(____AWPB_POP3Client))
  If *this
    *this\vt   = ?____AWPB_POP3Client_VT
    *this\Caps = ""
    *this\Conn = AllocateMemory(SizeOf(____NW_NetworkConn))
    If *this\Conn
    	InitializeStructure(*this\Conn, ____NW_NetworkConn)
    	*this\Conn\EnableLog = EnableLog
    	*this\Conn\LogData   = ""
      *this\Conn\TimeOut   = 15000    ; TimeOut (Standard: 15 Sekunden)
      *this\Conn\Retries   = 3        ; Retries im Fehlerfall (standard: 3)
      *this\Conn\Buffer    = AllocateMemory(#____NW_BUF_SIZE + 2) ; Sende- und Empfangspuffer anfordern; max 65000 Bytes
      If *this\Conn\Buffer
        ProcedureReturn *this
      EndIf
      FreeMemory(*this\Conn)
    EndIf
    FreeMemory(*this)
  EndIf

  ProcedureReturn #Null
EndProcedure
Procedure.i ____AWPB_POP3Client_NOOP(*this.____AWPB_POP3Client)
  Protected res.i, retries.i, done.b

  retries = *this\Conn\Retries
  res     = #AWPB_POP3_ERR_FAIL
  done    = #False
  If *this\Conn\ConnID
    Repeat
      If ____NW_Execute(*this\Conn, "NOOP", *this\Conn\Retries)
        Select *this\Conn\ResultCode
          Case #____NW_CD_OK:   res = #AWPB_POP3_ERR_NONE : done = #True
          Case #____NW_CD_ERR:  res = #AWPB_POP3_ERR_FAIL : done = #True
          Default:              res = #AWPB_POP3_ERR_FAIL : retries - 1
        EndSelect
      Else
        retries - 1
      EndIf
    Until done Or (retries <= 0)
  EndIf

  ProcedureReturn res
EndProcedure
Procedure.i ____AWPB_POP3Client_RSET(*this.____AWPB_POP3Client)
  Protected res.i, retries.i, done.b

  retries = *this\Conn\Retries
  res     = #AWPB_POP3_ERR_FAIL
  done    = #False
  If *this\Conn\ConnID
    Repeat
      If ____NW_Execute(*this\Conn, "RSET", *this\Conn\Retries)
        Select *this\Conn\ResultCode
          Case #____NW_CD_OK:   res = #AWPB_POP3_ERR_NONE : done = #True
          Case #____NW_CD_ERR:  res = #AWPB_POP3_ERR_FAIL : done = #True
          Default:              res = #AWPB_POP3_ERR_FAIL : retries - 1
        EndSelect
      Else
        retries - 1
      EndIf
    Until done Or (retries <= 0)
  EndIf

  ProcedureReturn res
EndProcedure
Procedure.i ____AWPB_POP3Client_DELE(*this.____AWPB_POP3Client, index.i)
  Protected res.i, retries.i, done.b

  retries = *this\Conn\Retries
  res     = #AWPB_POP3_ERR_FAIL
  done    = #False
  If *this\Conn\ConnID
    Repeat
      If ____NW_Execute(*this\Conn, "DELE " + Str(index), *this\Conn\Retries)
        Select *this\Conn\ResultCode
          Case #____NW_CD_OK:   res = #AWPB_POP3_ERR_NONE : done = #True
          Case #____NW_CD_ERR:  res = #AWPB_POP3_ERR_FAIL : done = #True
          Default:             res = #AWPB_POP3_ERR_FAIL : retries - 1
        EndSelect
      Else
        retries - 1
      EndIf
    Until done Or (retries <= 0)
  EndIf

  ProcedureReturn res
EndProcedure
Procedure.s ____AWPB_POP3Client_CAPA(*this.____AWPB_POP3Client)
  Protected retries.i, done.b, result.s

  retries = *this\Conn\Retries
  result  = ""
  done    = #False
  If *this\Conn\ConnID
    Repeat
      If ____NW_Execute(*this\Conn, "CAPA", *this\Conn\Retries)
        Select *this\Conn\ResultCode
          Case #____NW_CD_OK:   done = #True : ____NW_ReadData(*this\Conn) : result = *this\Conn\DataString
          Case #____NW_CD_ERR:  result = "" : done = #True
          Default:              result = "" : retries - 1
        EndSelect
      Else
        retries - 1
      EndIf
    Until done Or (retries <= 0)
  EndIf

  ProcedureReturn result
EndProcedure
Procedure.i ____AWPB_POP3Client_STAT(*this.____AWPB_POP3Client)
  Protected res.i, retries.i, done.b

  retries = *this\Conn\Retries
  res     = -1
  done    = #False
  If *this\Conn\ConnID
    Repeat
      If ____NW_Execute(*this\Conn, "STAT", *this\Conn\Retries)
        Select *this\Conn\ResultCode
          Case #____NW_CD_OK:   res = Val(StringField(*this\Conn\ResultString, 2, " ")) : done = #True
          Case #____NW_CD_ERR:  res = -1 : done = #True
          Default:             res = -1 : retries - 1
        EndSelect
      Else
        retries - 1
      EndIf
    Until done Or (retries <= 0)
  EndIf

  ProcedureReturn res
EndProcedure
Procedure.s ____AWPB_POP3Client_TOP(*this.____AWPB_POP3Client, index.i)
  Protected retries.i, done.b, result.s

  retries = *this\Conn\Retries
  result  = ""
  done    = #False
  If *this\Conn\ConnID
    Repeat
      If ____NW_Execute(*this\Conn, "TOP " + Str(index) + " 0", *this\Conn\Retries)
        Select *this\Conn\ResultCode
          Case #____NW_CD_OK:   done = #True : ____NW_ReadData(*this\Conn) : result = *this\Conn\DataString
          Case #____NW_CD_ERR:  result = "" : done = #True
          Default:             result = "" : retries - 1
        EndSelect
      Else
        retries - 1
      EndIf
    Until done Or (retries <= 0)
  EndIf

  ProcedureReturn result
EndProcedure
Procedure.s ____AWPB_POP3Client_RETR(*this.____AWPB_POP3Client, index.i)
  Protected retries.i, done.b, result.s

  retries = *this\Conn\Retries
  result  = ""
  done    = #False
  If *this\Conn\ConnID
    Repeat
      If ____NW_Execute(*this\Conn, "RETR " + Str(index), *this\Conn\Retries)
      	Select *this\Conn\ResultCode
          Case #____NW_CD_OK:   done = #True : ____NW_ReadData(*this\Conn) : result = *this\Conn\DataString
          Case #____NW_CD_ERR:  result = "" : done = #True
          Default:              result = "" : retries - 1
        EndSelect
      Else
        retries - 1
      EndIf
    Until done Or (retries <= 0)
  EndIf

  ProcedureReturn result
EndProcedure
Procedure.s ____AWPB_POP3Client_LIST(*this.____AWPB_POP3Client, index.i)
  Protected retries.i, done.b, result.s, res.b

  retries = *this\Conn\Retries
  result  = ""
  done    = #False
  If *this\Conn\ConnID
    Repeat
      If index
        res = ____NW_Execute(*this\Conn, "LIST " + Str(index), *this\Conn\Retries)
      Else
        ____NW_Execute(*this\Conn, "LIST", *this\Conn\Retries)
      EndIf
      If res
        Select *this\Conn\ResultCode
          Case #____NW_CD_OK:   done = #True : result = *this\Conn\ResultString
          Case #____NW_CD_ERR:  result = ""  : done = #True
          Default:              result = ""  : retries - 1
        EndSelect
      Else
        retries - 1
      EndIf
    Until done Or (retries <= 0)
  EndIf

  ProcedureReturn result
EndProcedure
Procedure.i ____AWPB_POP3Client_QUIT(*this.____AWPB_POP3Client)
  Protected res.i, retries.i, done.b

  retries = *this\Conn\Retries
  res     = #AWPB_POP3_ERR_FAIL
  done    = #False
  If *this\Conn\ConnID
    Repeat
      If ____NW_Execute(*this\Conn, "QUIT", *this\Conn\Retries)
        Select *this\Conn\ResultCode
          Case #____NW_CD_OK:   res = #AWPB_POP3_ERR_NONE : done = #True
          Case #____NW_CD_ERR:  res = #AWPB_POP3_ERR_FAIL : done = #True
          Default:              res = #AWPB_POP3_ERR_FAIL : retries - 1
        EndSelect
      Else
        retries - 1
      EndIf
    Until done Or (retries <= 0)
  EndIf

  ProcedureReturn res
EndProcedure
Procedure.i ____AWPB_POP3Client_USER(*this.____AWPB_POP3Client)
  Protected res.i, retries.i, done.b

  retries = *this\Conn\Retries
  res     = #AWPB_POP3_ERR_FAIL
  done    = #False
  If *this\Conn\ConnID
    Repeat
      If ____NW_Execute(*this\Conn, "USER " + *this\Conn\User, *this\Conn\Retries)
        Select *this\Conn\ResultCode
          Case #____NW_CD_OK:   res = #AWPB_POP3_ERR_NONE : done = #True
          Case #____NW_CD_ERR:  res = #AWPB_POP3_ERR_FAIL : done = #True
          Default:              res = #AWPB_POP3_ERR_FAIL : retries - 1
        EndSelect
      Else
        retries - 1
      EndIf
    Until done Or (retries <= 0)
  EndIf

  ProcedureReturn res
EndProcedure
Procedure.i ____AWPB_POP3Client_PASS(*this.____AWPB_POP3Client)
  Protected res.i, retries.i, done.b

  retries = *this\Conn\Retries
  res     = #AWPB_POP3_ERR_FAIL
  done    = #False
  If *this\Conn\ConnID
    Repeat
      If ____NW_Execute(*this\Conn, "PASS " + *this\Conn\Pass, *this\Conn\Retries)
        Select *this\Conn\ResultCode
          Case #____NW_CD_OK:   res = #AWPB_POP3_ERR_NONE : done = #True
          Case #____NW_CD_ERR:  res = #AWPB_POP3_ERR_FAIL : done = #True
          Default:              res = #AWPB_POP3_ERR_FAIL : retries - 1
        EndSelect
      Else
        retries - 1
      EndIf
    Until done Or (retries <= 0)
  EndIf

  ProcedureReturn res
EndProcedure
Procedure.i ____AWPB_POP3Client_Login(*this.____AWPB_POP3Client)
  Protected res.i

  res = ____AWPB_POP3Client_USER(*this)
  If res = #AWPB_POP3_ERR_NONE
    res = ____AWPB_POP3Client_PASS(*this)
  EndIf

  ProcedureReturn res
EndProcedure
Procedure.i ____AWPB_POP3Client_APOP(*this.____AWPB_POP3Client)
  Protected res.i, digest.s, retries.i, done.b

  res = #AWPB_POP3_ERR_FAIL
  If *this\Conn\ConnID
    If *this\TimeStamp
      digest  = ____SUP_MD5Str(*this\TimeStamp + *this\Conn\Pass)
      retries = *this\Conn\Retries
      done    = #False
      Repeat
        If ____NW_Execute(*this\Conn, "APOP " + *this\Conn\User + " " + digest, *this\Conn\Retries)
          Select *this\Conn\ResultCode
            Case #____NW_CD_OK:   res = #AWPB_POP3_ERR_NONE : done = #True
            Case #____NW_CD_ERR:  res = #AWPB_POP3_ERR_FAIL : done = #True
            Default:              res = #AWPB_POP3_ERR_FAIL : retries - 1
          EndSelect
        Else
          retries - 1
        EndIf
      Until done Or (retries <= 0)
    EndIf
  EndIf

  ProcedureReturn res
EndProcedure
Procedure.i ____AWPB_POP3Client_STLS(*this.____AWPB_POP3Client)
  Protected res.i, retries.i, done.b

  res = #AWPB_POP3_ERR_FAIL
  If *this\Conn\ConnID
    CompilerIf Defined(Enable_OpenSSL, #PB_Constant)
      retries = *this\Conn\Retries
      done    = #False
      Repeat
        If ____NW_Execute(*this\Conn, "STLS", *this\Conn\Retries)
          Select *this\Conn\ResultCode
            Case #____NW_CD_OK:     res = #AWPB_POP3_ERR_NONE : done = #True
            Case #____NW_CD_ERR:    res = #AWPB_POP3_ERR_FAIL : done = #True
            Default: 						    res = #AWPB_POP3_ERR_FAIL : retries - 1
          EndSelect
        Else
          retries - 1
        EndIf
      Until done Or (retries <= 0)

      If res = #AWPB_POP3_ERR_NONE
      	If ____NW_SSL_Connect(*this\conn, #____AWPB_SSL_METHOD_V23) <> #____AWPB_SSL_ERR_NONE
      		res = #AWPB_POP3_ERR_FAIL
      	EndIf
      EndIf

    CompilerEndIf
  EndIf

  ProcedureReturn res
EndProcedure
Procedure.i ____AWPB_POP3Client_Connect(*this.____AWPB_POP3Client, flags.i)
  Protected res.i, tsb.i, tse.i

  res               = #AWPB_POP3_ERR_FAIL
  *this\Caps        = ""
  *this\TimeStamp   = ""
  *this\Conn\ConnID = OpenNetworkConnection(*this\Conn\Host, *this\Conn\Port)
  If *this\Conn\ConnID
    If ____NW_ReadResult(*this\Conn) = #____NW_CD_OK
      tsb = FindString(*this\Conn\ResultString, "<", 1)        ; Timestamp aus Serverbegrung extrahieren; ggf fr APOP ntig
      If tsb
        tse = ____MSG_FindLastChar(*this\Conn\ResultString, '>')
        If tse > tsb
          *this\TimeStamp = Mid(*this\Conn\ResultString, tsb, tse - tsb + 1)
        EndIf
      EndIf
      *this\Caps = ____AWPB_POP3Client_CAPA(*this)
      CompilerIf Defined(Enable_OpenSSL, #PB_Constant)
      	*this\conn\TLSactive = #False
       	If AWPB_SSL_Available()
	      	If FindString(*this\Caps, "STLS", 1, #PB_String_NoCase) > 0
		      	If (flags & #AWPB_POP3_CONN_TLS) = #AWPB_POP3_CONN_TLS
							; TLS starten; OpenSSL erforderlich [hierzu #Enable_OpenSSL VOR include der datei setzen]
	          	*this\Caps = ""
	          	res        = ____AWPB_POP3Client_STLS(*this)
	          	If res = #AWPB_POP3_ERR_NONE
	          		*this\conn\TLSactive = #True
	            	*this\caps = ____AWPB_POP3Client_CAPA(*this)
	          	EndIf
	          EndIf
	        EndIf
        EndIf
      CompilerEndIf

      res = #AWPB_POP3_ERR_FAIL
      If ((flags & #AWPB_POP3_CONN_APOP) = #AWPB_POP3_CONN_APOP) And (FindString(*this\Caps, "APOP", 1) > 0)  ; falls noch nicht eingeloggt, nchste unsicherere methode versuchen
      	res = ____AWPB_POP3Client_APOP(*this)
      EndIf
      If (res <> #AWPB_POP3_ERR_NONE) And ((flags & #AWPB_POP3_CONN_USER) = #AWPB_POP3_CONN_USER) And (FindString(*this\Caps, "USER", 1) > 0)   ; falls noch nicht eingeloggt, nchste unsicherere methode versuchen
        res = ____AWPB_POP3Client_Login(*this)
      EndIf
      If (res <> #AWPB_POP3_ERR_NONE) And ((flags & #AWPB_POP3_CONN_AUTO) = #AWPB_POP3_CONN_AUTO)
        res = ____AWPB_POP3Client_STAT(*this)
        If res >= 0
          res = #AWPB_POP3_ERR_NONE
        Else
          res = #AWPB_POP3_ERR_FAIL
        EndIf
      EndIf
    EndIf
    If res <> #AWPB_POP3_ERR_NONE                   ; nicht eingeloggt? Verbindung komplett trennen
      CloseNetworkConnection(*this\Conn\ConnID)
      *this\Conn\ConnID = #Null
    EndIf
  EndIf

  ProcedureReturn res
EndProcedure
Procedure.s ____AWPB_POP3Client_GetServerResult(*this.____AWPB_POP3Client)
  ProcedureReturn *this\Conn\ResultString
EndProcedure
Procedure   ____AWPB_POP3Client_SetServer(*this.____AWPB_POP3Client, Host.s, Port.i)
  *this\Conn\Host = Host
  *this\Conn\Port = Port
EndProcedure
Procedure   ____AWPB_POP3Client_SetLogin(*this.____AWPB_POP3Client, User.s, Pass.s)
  *this\Conn\User = User
  *this\Conn\Pass = Pass
EndProcedure
Procedure   ____AWPB_POP3Client_SetTimeOut(*this.____AWPB_POP3Client, TimeOut.i)
  *this\Conn\Timeout = TimeOut
EndProcedure
Procedure.i ____AWPB_POP3Client_GetMailCount(*this.____AWPB_POP3Client)
  ProcedureReturn ____AWPB_POP3Client_STAT(*this)
EndProcedure
Procedure.s ____AWPB_POP3Client_GetMailHeaderStr(*this.____AWPB_POP3Client, index.i)
  ProcedureReturn ____AWPB_POP3Client_TOP(*this, index)
EndProcedure
Procedure.i ____AWPB_POP3Client_GetMailHeader(*this.____AWPB_POP3Client, index.i)
  Protected *msg.AWPB_Message, src.s

  *msg = AWPB_CreateMessageObj()
  If *msg
    src = ____AWPB_POP3Client_TOP(*this, index)
    If src
      If *msg\SetMailSource(src)
        ProcedureReturn *msg
      EndIf
    EndIf
    *msg\Destroy()
  EndIf

  ProcedureReturn #Null
EndProcedure
Procedure.s ____AWPB_POP3Client_GetMailStr(*this.____AWPB_POP3Client, index.i)
  ProcedureReturn ____AWPB_POP3Client_RETR(*this, index)
EndProcedure
Procedure.i ____AWPB_POP3Client_GetMail(*this.____AWPB_POP3Client, index.i)
  Protected *msg.AWPB_Message, src.s

  *msg = AWPB_CreateMessageObj()
  If *msg
    src = ____AWPB_POP3Client_RETR(*this, index)
    If src
      If *msg\SetMailSource(src)
        ProcedureReturn *msg
      EndIf
    EndIf
    *msg\Destroy()
  EndIf

  ProcedureReturn #Null
EndProcedure
Procedure.i ____AWPB_POP3Client_DisConnect(*this.____AWPB_POP3Client)
  Protected res.i

  res = #AWPB_POP3_ERR_FAIL
  If *this\Conn\ConnID
    res = ____AWPB_POP3Client_QUIT(*this)
    If res = #AWPB_POP3_ERR_NONE
      CloseNetworkConnection(*this\Conn\ConnID)
      *this\Conn\ConnID = #Null
    EndIf
  EndIf

  ProcedureReturn res
EndProcedure
Procedure.i ____AWPB_POP3Client_Destroy(*this.____AWPB_POP3Client)
  If *this\Conn
    ____AWPB_POP3Client_DisConnect(*this)
    If *this\Conn\Buffer : FreeMemory(*this\Conn\Buffer) : EndIf
    FreeMemory(*this\Conn)
  EndIf
  FreeMemory(*this)
  ProcedureReturn #Null
EndProcedure
Procedure.s ____AWPB_POP3Client_GetLog(*this.____AWPB_POP3Client)
	If *this\Conn
		ProcedureReturn *this\Conn\LogData
	EndIf

	ProcedureReturn #NULL$
EndProcedure
Procedure   ____AWPB_POP3Client_ClearLog(*this.____AWPB_POP3Client)
	If *this\Conn
		*this\Conn\LogData = ""
	EndIf
EndProcedure
Procedure   ____AWPB_POP3Client_EnableLog(*this.____AWPB_POP3Client)
	If *this\Conn
		*this\Conn\EnableLog = #True
	EndIf
EndProcedure
Procedure   ____AWPB_POP3Client_DisableLog(*this.____AWPB_POP3Client)
	If *this\Conn
		*this\Conn\EnableLog = #False
	EndIf
EndProcedure
;}
DataSection ;{ Virtual Function Table
____AWPB_POP3Client_VT:
  ; public
  Data.i @____AWPB_POP3Client_Destroy()
  Data.i @____AWPB_POP3Client_NOOP()
  Data.i @____AWPB_POP3Client_RSET()
  Data.i @____AWPB_POP3Client_Connect()
  Data.i @____AWPB_POP3Client_GetServerResult()
  Data.i @____AWPB_POP3Client_SetServer()
  Data.i @____AWPB_POP3Client_SetLogin()
  Data.i @____AWPB_POP3Client_SetTimeOut()
  Data.i @____AWPB_POP3Client_DisConnect()
  Data.i @____AWPB_POP3Client_GetMailCount()
  Data.i @____AWPB_POP3Client_GetMailHeaderStr()
  Data.i @____AWPB_POP3Client_GetMailHeader()
  Data.i @____AWPB_POP3Client_GetMailStr()
  Data.i @____AWPB_POP3Client_GetMail()
	Data.i @____AWPB_POP3Client_GetLog()
  Data.i @____AWPB_POP3Client_ClearLog()
	Data.i @____AWPB_POP3Client_EnableLog()
	Data.i @____AWPB_POP3Client_DisableLog()

  ; private
  Data.i @____AWPB_POP3Client_CAPA()
  Data.i @____AWPB_POP3Client_LIST()
  Data.i @____AWPB_POP3Client_QUIT()
  Data.i @____AWPB_POP3Client_USER()
  Data.i @____AWPB_POP3Client_PASS()
  Data.i @____AWPB_POP3Client_Login()
  Data.i @____AWPB_POP3Client_APOP()
  Data.i @____AWPB_POP3Client_STLS()
  Data.i @____AWPB_POP3Client_DELE()
  Data.i @____AWPB_POP3Client_STAT()
  Data.i @____AWPB_POP3Client_TOP()
  Data.i @____AWPB_POP3Client_RETR()

EndDataSection ;}
;}

